@@ -19,6 +19,8 @@ from .utils import UseTlsAdapter, send_response
 
 requests.packages.urllib3.disable_warnings()
 log = logging.getLogger(__name__)
+log.setLevel(logging.DEBUG)
+logging.basicConfig()
 
 
 class HttpClient(object):
@@ -27,8 +29,10 @@ class HttpClient(object):
     ip_address = None
     endpoint = None
 
-    def __init__(self, tls_version=utils.DEFAULT_TLS_VERSION, cert=None):
+    def __init__(self, tls_version=utils.DEFAULT_TLS_VERSION, cert=None,
+                 user=None, password=None):
         self.cert = cert
+        self.auth = (user, password)
         self._tls_version = tls_version
         self._tls_session = Session()
         self._tls_session.mount("http://", UseTlsAdapter(self._tls_version))
@@ -68,7 +72,8 @@ class HttpClient(object):
                                                          verify=False,
                                                          data=payload,
                                                          headers=headers,
-                                                         cert=self.cert)
+                                                         cert=self.cert,
+                                                         auth=self.auth)
             else:
                 response = self._tls_session.request(http_method,
                                                      cmd_url,
@@ -76,7 +81,8 @@ class HttpClient(object):
                                                      timeout=utils.TIMEOUT,
                                                      verify=False,
                                                      headers=headers,
-                                                     cert=self.cert)
+                                                     cert=self.cert,
+                                                     auth=self.auth)
             self._tls_session.close()
 
             # Raise specific error for authentication failure
@@ -141,6 +147,7 @@ class AccessInfoMixin(object):
     endpoint = None
     ip_address = None
     cert = None
+    auth = None
 
     @property
     def access_info(self):
@@ -148,6 +155,8 @@ class AccessInfoMixin(object):
             "endpoint": self.endpoint,
             "ip_address": self.ip_address,
             "cert": self.cert,
+            "auth": self.auth,
+            "appliance": self
         }
         return info
 
@@ -167,7 +176,7 @@ class BaseKempObject(HttpClient, AccessInfoMixin):
     _API_IGNORE = (
         "log_urls", "ip_address", "endpoint", "rsindex", "vsindex", "index",
         "status", "subvs_data", "subvs_entries", "real_servers", "cert",
-        "checkuse1_1", "mastervsid", "API_INIT_PARAMS", "API_TAG"
+        "checkuse1_1", "mastervsid", "API_INIT_PARAMS", "API_TAG", "auth"
     )
 
     def __init__(self, loadmaster_info, **kwargs):
@@ -181,7 +190,12 @@ class BaseKempObject(HttpClient, AccessInfoMixin):
         except KeyError:
             raise GenericObjectMissingLoadMasterInfo(type(self), "ip_address")
 
-        super(BaseKempObject, self).__init__(loadmaster_info)
+        try:
+            self.auth = loadmaster_info["auth"]
+        except KeyError:
+            raise GenericObjectMissingLoadMasterInfo(type(self), "auth")
+        super(BaseKempObject, self).__init__(cert=self.cert,
+                                             user=self.auth[0], password=self.auth[1])
 
     def __repr__(self):
         return '{} {}'.format(
@@ -257,9 +271,14 @@ class BaseKempObject(HttpClient, AccessInfoMixin):
 
     def populate_default_attributes(self, params):
         """Populate object instance with standard defaults"""
-        if self.API_TAG in params.keys():
-            self.populate_default_attributes(params[self.API_TAG])
+        if len(params) == 0:
+            log.warning("No data was returned, leaving data intact")
             return
 
+        if len(params) == 1:
+            if self.API_TAG in params.keys():
+                self.populate_default_attributes(params[self.API_TAG])
+                return
+
         for attribute, tag in self._API_DEFAULT_ATTRIBUTES.items():
             setattr(self, attribute, params.get(tag, None))
