@@ -25,16 +25,18 @@ from python_kemptech_api.exceptions import (
 from python_kemptech_api.generic import HttpClient, AccessInfoMixin
 from python_kemptech_api.objects import (
     VirtualService,
+    CipherSet,
     Certificate,
     Sso,
     Rule,
+    GlobalACL,
     Template,
     Fqdn,
     Cluster,
     Range,
     CustomLocation,
-    Interface
-)
+    Interface,
+    IntermediateCertificate)
 from python_kemptech_api.utils import (
     send_response,
     validate_port,
@@ -62,7 +64,9 @@ class BaseKempAppliance(HttpClient, AccessInfoMixin):
         self.version = None
 
         super(BaseKempAppliance, self).__init__(utils.DEFAULT_TLS_VERSION,
-                                                self.cert)
+                                                self.cert,
+                                                user=self.username,
+                                                password=self.password)
 
     def __repr__(self):
         return '{}:{}'.format(self.ip_address, self.port)
@@ -113,20 +117,7 @@ class BaseKempAppliance(HttpClient, AccessInfoMixin):
 
     @property
     def endpoint(self):
-        if self.cert:
-            return "https://{ip}:{port}/{access}".format(
-                ip=self.ip_address,
-                port=self.port,
-                access=self.access_point
-            )
-        else:
-            return "https://{user}:{pw}@{ip}:{port}/{access}".format(
-                user=self.username,
-                pw=self.password,
-                ip=self.ip_address,
-                port=self.port,
-                access=self.access_point
-            )
+        return "https://{}:{}/{}".format(self.ip_address, self.port, self.access_point)
 
     @property
     def capabilities(self):
@@ -144,6 +135,8 @@ class BaseKempAppliance(HttpClient, AccessInfoMixin):
         command = '{}/{}'.format(self.endpoint, command)
         if self.cert:
             curl.extend(['-E', self.cert])
+        else:
+            curl.extend(['-u', '{}:{}'.format(self.username, self.password)])
         curl.append(command)
         return curl
 
@@ -191,7 +184,7 @@ class BaseKempAppliance(HttpClient, AccessInfoMixin):
 
         :return: Dict with interface name and Interface instance key / value pairs.
         """
-        return {interface.name: interface
+        return {interface.interface: interface
                 for interface in self.get_interfaces()}
 
     def __getitem__(self, parameter):
@@ -219,6 +212,13 @@ class BaseKempAppliance(HttpClient, AccessInfoMixin):
         if not is_successful(response):
             raise LoadMasterParameterError(self, parameters)
 
+    @property
+    def acl(self):
+        return self.get_global_acl()
+
+    def get_global_acl(self):
+        return GlobalACL(self.access_info)
+
     def get_parameter(self, parameter):
         """Get the value of the given LoadMaster parameter
 
@@ -239,7 +239,7 @@ class BaseKempAppliance(HttpClient, AccessInfoMixin):
                             sorted(value.items()))
 
         if parameter == "version":
-            value = self.version = ".".join(value.split(".")[:3])
+            self.version = ".".join(value.split(".")[:3])
 
         return value
 
@@ -299,6 +299,20 @@ class BaseKempAppliance(HttpClient, AccessInfoMixin):
         self.version = None
         return is_successful(response)
 
+    def reset_logs(self):
+        response = self._get("/logging/resetlogs")
+        return is_successful(response)
+
+    def download_logs(self, filename=None):
+        response = self._get("/logging/downloadlogs")
+
+        filename = filename or "LoadMaster_" + self.ip_address + "_Logs.tgz"
+
+        with open(filename, 'w') as f:
+            f.write(response)
+
+        return filename
+
     def change_bal_password(self, new_password):
         parameters = {
             "currpassword": self.password,
@@ -693,7 +707,7 @@ class Geo(BaseKempAppliance):
                 v = None
             else:
                 try:
-                    v = 0 if not v else 1
+                    v = int(v)  # pylint: disable=redefined-variable-type
                 except ValueError:
                     pass
 
@@ -893,9 +907,13 @@ class Geo(BaseKempAppliance):
         return Fqdn(self.access_info, fqdn)
 
     def get_fqdns(self):
-        response = self._get("/listfqdns")
-        data = get_data(response)
-        return list_object(Fqdn, self.access_info, data)
+        try:
+            response = self._get("/listfqdns")
+            data = get_data(response)
+            return list_object(Fqdn, self.access_info, data)
+        except KempTechApiException:
+            # If API returns 'No geo data found' return an empty list
+            return []
 
     def get_fqdn(self, fqdn):
         service_id = {
@@ -1210,6 +1228,10 @@ class LoadMaster(BaseKempAppliance):
 
         return rule_object
 
+    def create_cipherset(self, cipherset_name, ciphers):
+        cipherset = CipherSet(self.access_info, cipherset_name, ciphers)
+        cipherset.save()
+
     def create_certificate(self, certificate, certfile, certpass=None):
         """Certificate factory with pre-configured LoadMaster connection."""
         cert = Certificate(self.access_info,
@@ -1218,6 +1240,13 @@ class LoadMaster(BaseKempAppliance):
                            certpass=certpass)
         return cert
 
+    def create_intermediate_certificate(self, certificate, certfile):
+        """Certificate factory with pre-configured LoadMaster connection."""
+        cert = IntermediateCertificate(self.access_info,
+                                       certificate,
+                                       certfile=certfile)
+        return cert
+
     def get_certificates(self, type='cert'):
         response = self._get("/listcert")
         data = get_data(response)
@@ -1226,18 +1255,40 @@ class LoadMaster(BaseKempAppliance):
         if not isinstance(certs, list):
             certs = [certs]
         for cert in certs:
-            certificate = self.build_certificate(cert, type=type)
+            certificate = self.build_certificate(cert)
+            certificates.append(certificate)
+        return certificates
+
+    def get_intermediate_certificates(self, type='cert'):
+        response = self._get("/listintermediate")
+        data = get_data(response)
+        certificates = []
+        certs = data.get('cert', [])
+        if not isinstance(certs, list):
+            certs = [certs]
+        for cert in certs:
+            certificate = self.build_intermediate_certificate(cert)
             certificates.append(certificate)
         return certificates
 
     def build_certificate(self, certificate, certfile=None,
-                          certpass=None, type='cert'):
+                          certpass=None):
         """Create a Certificte instance named certificate"""
         if certfile is not None:
             cert = Certificate(self.access_info, certificate,
-                               certfile=certfile, certpass=certpass, type=type)
+                               certfile=certfile, certpass=certpass)
         else:
-            cert = Certificate(self.access_info, certificate, type=type)
+            cert = Certificate(self.access_info, certificate)
+        return cert
+
+    def build_intermediate_certificate(self, certificate, certfile=None):
+        """Create a Certificte instance named certificate"""
+        if certfile is not None:
+            cert = IntermediateCertificate(self.access_info,
+                                           certificate,
+                                           certfile=certfile)
+        else:
+            cert = IntermediateCertificate(self.access_info, certificate)
         return cert
 
     def create_virtual_service(self, ip, port=80, protocol="tcp"):
@@ -1466,3 +1517,7 @@ class LoadMaster(BaseKempAppliance):
     def get_sdn_info(self):
         response = self._get('/sdninfo')
         return send_response(response)
+
+
+class LoadMasterGeo(LoadMaster, Geo):
+    pass
