@@ -678,6 +678,11 @@ class RealServer(BaseKempObject):
 
     def save(self, update=False):
         super(RealServer, self).save(update)
+        # If a new port is set, update the assigned port value
+        # in order to correctly look up the updated RS
+        # If .save fails, this will never be performed
+        if getattr(self, 'newport', None):
+            self.rsport = self.newport
         self.refresh()
 
 
